import React, { useContext, memo, useState, useCallback, useEffect, useMemo, useRef } from 'react';
import { Linking } from 'react-native';
import { SocketContextProvider, SocketContext } from './Context';
import { io } from 'socket.io-client';
import { useChatConfigurations } from '../ChatConfigurations';
import { From, MessageTypes, CarouselDestinationTypes } from '../../types/Message';
import { getHourAndMinutes } from '../../utils/getHourAndMinutes';
const SocketComponent = props => {
  const channel = 'webchat';
  const socketUrl = 'https://e118-191-193-237-58.ngrok.io';
  const {
    children,
    botId
  } = props;
  const {
    setBotConfigs
  } = useChatConfigurations();
  const clientRef = useRef(null);
  const [messages, setMessages] = useState([]);
  const [sessionId, setSessionId] = useState(null);
  const addMessage = useCallback(message => {
    setMessages(prevState => [...(prevState.filter(m => m.type !== MessageTypes.TYPING) ?? []), {
      ...message,
      hour: getHourAndMinutes()
    }]);
  }, []);
  const handleSubmitMessage = useCallback(_ref => {
    var _clientRef$current;
    let {
      message,
      type,
      ext
    } = _ref;
    const isMedia = type !== MessageTypes.TEXT;
    clientRef === null || clientRef === void 0 ? void 0 : (_clientRef$current = clientRef.current) === null || _clientRef$current === void 0 ? void 0 : _clientRef$current.emit('message', {
      botId: botId,
      message,
      isMedia,
      ext,
      id: sessionId,
      botChannel: 'WebChat'
    });
    setMessages(prevState => [...prevState, {
      from: From.USER,
      message,
      type,
      hour: getHourAndMinutes()
    }]);
  }, [sessionId]);
  const handleStartConversation = useCallback(_sessionId => {
    var _clientRef$current2;
    setSessionId(_sessionId);
    clientRef === null || clientRef === void 0 ? void 0 : (_clientRef$current2 = clientRef.current) === null || _clientRef$current2 === void 0 ? void 0 : _clientRef$current2.emit('start', {
      botId: botId,
      id: _sessionId,
      channel: channel,
      botChannel: 'WebChat',
      contactNumber: sessionId
    });
  }, [clientRef]);
  const startNewSession = client => {
    client.emit('subscribe', {
      botId: botId,
      botChannel: 'WebChat'
    });
  };
  const handleOpenLink = useCallback(async url => {
    await Linking.openURL(url);
  }, []);
  const handleCarouselButtonClick = _ref2 => {
    var _clientRef$current3;
    let {
      clickedButton,
      clickedCard
    } = _ref2;
    const {
      destination
    } = clickedButton;
    if ((destination === null || destination === void 0 ? void 0 : destination.type) === CarouselDestinationTypes.URL) {
      handleOpenLink(destination === null || destination === void 0 ? void 0 : destination.value);
    }
    if ((destination === null || destination === void 0 ? void 0 : destination.type) === CarouselDestinationTypes.PHONE) {
      var _destination$value;
      const treated = destination === null || destination === void 0 ? void 0 : (_destination$value = destination.value) === null || _destination$value === void 0 ? void 0 : _destination$value.replace(/[^A-Z0-9]/gi, '');
      handleOpenLink(`tel:+55${treated}`);
    }
    clientRef === null || clientRef === void 0 ? void 0 : (_clientRef$current3 = clientRef.current) === null || _clientRef$current3 === void 0 ? void 0 : _clientRef$current3.emit('action', {
      botId: botId,
      action: {
        type: MessageTypes.CAROUSEL,
        data: {
          clickedButton,
          clickedCard
        }
      },
      id: sessionId
    });
  };
  useEffect(() => {
    if (!clientRef.current) {
      const client = io(socketUrl);
      clientRef.current = client;
      client.on('connect', () => {
        if (!sessionId) {
          startNewSession(client);
        }
      });
      client.on('message', data => {
        addMessage({
          ...data,
          from: From.BOT
        });
      });
      client.on('subscribe-response', data => {
        var _data$bot, _data$bot$channels, _data$bot2, _webchatChannel$setti, _webchatChannel$setti2, _webchatChannel$setti3, _webchatChannel$setti4, _webchatChannel$setti5;
        setSessionId(data === null || data === void 0 ? void 0 : data.id);
        handleStartConversation(data.id);
        const [webchatChannel] = data === null || data === void 0 ? void 0 : (_data$bot = data.bot) === null || _data$bot === void 0 ? void 0 : (_data$bot$channels = _data$bot.channels) === null || _data$bot$channels === void 0 ? void 0 : _data$bot$channels.filter(ch => {
          return ch.channelId === 'WebChat';
        });
        const botConfigs = {
          title: data === null || data === void 0 ? void 0 : (_data$bot2 = data.bot) === null || _data$bot2 === void 0 ? void 0 : _data$bot2.title,
          botFab: webchatChannel === null || webchatChannel === void 0 ? void 0 : (_webchatChannel$setti = webchatChannel.settings) === null || _webchatChannel$setti === void 0 ? void 0 : _webchatChannel$setti.botFab,
          colors: {
            main: webchatChannel === null || webchatChannel === void 0 ? void 0 : (_webchatChannel$setti2 = webchatChannel.settings) === null || _webchatChannel$setti2 === void 0 ? void 0 : _webchatChannel$setti2.mainColor,
            secondary: webchatChannel === null || webchatChannel === void 0 ? void 0 : (_webchatChannel$setti3 = webchatChannel.settings) === null || _webchatChannel$setti3 === void 0 ? void 0 : _webchatChannel$setti3.secondaryColor,
            mainText: webchatChannel === null || webchatChannel === void 0 ? void 0 : (_webchatChannel$setti4 = webchatChannel.settings) === null || _webchatChannel$setti4 === void 0 ? void 0 : _webchatChannel$setti4.mainTextColor,
            secondaryText: webchatChannel === null || webchatChannel === void 0 ? void 0 : (_webchatChannel$setti5 = webchatChannel.settings) === null || _webchatChannel$setti5 === void 0 ? void 0 : _webchatChannel$setti5.secondaryTextColor
          }
        };
        setBotConfigs(botConfigs);
      });
      client.on('reconnect_attempt', () => {
        client.io.opts.transports = ['polling', 'websocket'];
      });
      client.on('disconnect', () => {
        if (clientRef.current) {
          console.log('ws closed by server');
        } else {
          console.log('ws closed by app component unmount');
          return;
        }
      });
      client.on('end_conversation', data => {
        if (!(data !== null && data !== void 0 && data.isTransfer)) {
          setSessionId('');
        }
      });
      return () => {
        clientRef.current = null;
        client.close();
      };
    }
    return () => {
      clientRef.current = null;
    };
  }, [botId, handleStartConversation]);
  const state = useMemo(() => ({
    handleSubmitMessage,
    messages,
    handleCarouselButtonClick
  }), [messages]);
  return /*#__PURE__*/React.createElement(SocketContextProvider, {
    value: state
  }, children);
};
export const useSocketContext = () => {
  return useContext(SocketContext);
};
export default /*#__PURE__*/memo(SocketComponent);
//# sourceMappingURL=Component.js.map